@@ -594,6 +594,14 @@ EnzymeGradientUtilsErase(gutils, a) = ccall(
     gutils,
     a,
 )
+EnzymeReplaceOriginalToNew(gutils, orig, rep) = ccall(
+    (:EnzymeReplaceOriginalToNew, libEnzyme),
+    Cvoid,
+    (EnzymeGradientUtilsRef, LLVMValueRef, LLVMValueRef),
+    gutils,
+    orig,
+    rep
+)
 EnzymeGradientUtilsEraseWithPlaceholder(gutils, a, orig, erase) = ccall(
     (:EnzymeGradientUtilsEraseWithPlaceholder, libEnzyme),
     Cvoid,
@@ -884,7 +892,7 @@ EnzymeGradientUtilsSetReverseBlock(
     block,
 ) = ccall(
     (:EnzymeGradientUtilsSetReverseBlock, libEnzyme),
-    LLVM.API.LLVMBasicBlockRef,
+    Cvoid,
     (
         EnzymeGradientUtilsRef,
         LLVM.API.LLVMBasicBlockRef,
@@ -1324,7 +1332,9 @@ end
     ET_TypeDepthExceeded = 6,
     ET_MixedActivityError = 7,
     ET_IllegalReplaceFicticiousPHIs = 8,
-    ET_GetIndexError = 9
+    ET_GetIndexError = 9,
+    ET_NoTruncate = 10,
+    ET_GCRewrite = 11
 )
 
 function EnzymeTypeAnalyzerToString(typeanalyzer)
