@@ -480,6 +480,8 @@ const generic_method_offsets = Dict{String,Tuple{Int,Int}}((
     "ijl_f__apply_latest" => (2, 3),
     "jl_f__call_latest" => (2, 3),
     "ijl_f__call_latest" => (2, 3),
+    "jl_f_invokelatest" => (2, 3),
+    "ijl_f_invokelatest" => (2, 3),
     "jl_f_invoke" => (2, 3),
     "jl_invoke" => (1, 3),
     "jl_apply_generic" => (1, 2),
@@ -616,6 +618,10 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
     method_table = Core.Compiler.method_table(interp)
     bt = backtrace(inst)
     dest = called_operand(inst)
+    
+    if isa(dest, LLVM.PHIInst) && all(Base.Fix1(==, operands(dest)[1]), operands(dest))
+       dest = operands(dest)[1]
+    end
     if isa(dest, LLVM.ConstantExpr) && opcode(dest) == LLVM.API.LLVMIntToPtr && isa(operands(dest)[1], LLVM.ConstantExpr) && opcode(operands(dest)[1]) == LLVM.API.LLVMPtrToInt
        dest = operands(operands(dest)[1])[1] 
     end
@@ -780,8 +786,8 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
             fname = ops[2]
 
             if isa(flib, LLVM.LoadInst)
-                op, _ = get_base_and_offset(operands(flib)[1]; offsetAllowed=false, inttoptr=true)
-                
+                op, off = get_base_and_offset(operands(flib)[1]; inttoptr=true)
+            
                 if isa(op, LLVM.LoadInst)
                     pop, _ = get_base_and_offset(operands(op)[1]; offsetAllowed=false, inttoptr=true)
 
@@ -790,7 +796,7 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
                 
                         rep = zop
                         PT = value_type(rep)
-                        if isa(PT, LLVM.PointerType)
+                        if isa(PT, LLVM.PointerType) 
                             rep = LLVM.const_inttoptr(rep, LLVM.PointerType(eltype(PT)))
                             rep = LLVM.const_addrspacecast(rep, PT)
                             replace_uses!(pop, rep)
@@ -802,7 +808,10 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
                 end
                         
                 if isa(op, ConstantInt)
-                    rep = reinterpret(Ptr{Cvoid}, convert(Csize_t, op) + 8)
+	    	    @static if VERSION < v"1.12"
+			off += 8
+		    end
+                    rep = reinterpret(Ptr{Cvoid}, convert(Csize_t, op) + off)
                     ld = unsafe_load(convert(Ptr{Ptr{Cvoid}}, rep))
                     flib = Base.unsafe_pointer_to_objref(ld)
                 end
@@ -824,8 +833,8 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
                fname = String(map(Base.Fix1(convert, UInt8), collect(fname)[1:(end-1)]))
             end
 
-            if !isa(fname, String) || !isa(flib, String)
-                return
+	    if !isa(fname, String) || !isa(flib, String)
+		return
             end
 
             found, replaceWith = try_import_llvmbc(mod, flib, fname, imported)
@@ -876,7 +885,7 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
                             fname,
                         )
                     end
-                catch
+                catch e
                     nothing
                 end
 
@@ -1144,7 +1153,7 @@ function check_ir!(interp, @nospecialize(job::CompilerJob), errors::Vector{IRErr
 		    else
 			false, nothing
 		    end
-
+	
                     lfn = nothing
                     if found 
                         lfn = replaceWith
