@@ -49,13 +49,17 @@ function get_shadow_type(gutils::GradientUtils, T::LLVM.LLVMType)
 end
 function get_uncacheable(gutils::GradientUtils, orig::LLVM.CallInst)
     uncacheable = Vector{UInt8}(undef, length(collect(LLVM.operands(orig))) - 1)
+    if get_mode(gutils) == API.DEM_ForwardMode
+        fill!(uncacheable, 0)
+        return uncacheable
+    end
     if API.EnzymeGradientUtilsGetUncacheableArgs(
         gutils,
         orig,
         uncacheable,
         length(uncacheable),
     ) != 1
-        uncacheable .= 1
+        fill!(uncacheable, 1)
     end
     return uncacheable
 end
@@ -95,7 +99,7 @@ function add_reverse_block!(gutils::GradientUtils, block::LLVM.BasicBlock, name:
 end
 
 function set_reverse_block!(gutils::GradientUtils, block::LLVM.BasicBlock)
-    return LLVM.BasicBlock(API.EnzymeGradientUtilsSetReverseBlock(gutils, block))
+    return API.EnzymeGradientUtilsSetReverseBlock(gutils, block)
 end
 
 function get_or_insert_conditional_execute!(fn::LLVM.Function; force_run=false, need_result=true, preprocess=nothing, postprocess=nothing, postprocess_const=nothing, cmpidx::Int = 1)
@@ -317,13 +321,14 @@ function batch_call_same_with_inverted_arg_if_active!(
     args::Vector{<:LLVM.Value},
     valTys::Vector{API.CValueType},
     lookup::Bool;
+    need_result = true,
     kwargs...
 )
 
     width = get_width(gutils)
 
     void_rt = value_type(orig) ==LLVM.VoidType()
-    shadow = if !void_rt
+    shadow = if !void_rt && need_result
         ST = LLVM.LLVMType(API.EnzymeGetShadowType(width, value_type(orig)))
         LLVM.UndefValue(ST)::LLVM.Value
     end
@@ -339,7 +344,7 @@ function batch_call_same_with_inverted_arg_if_active!(
                 end
             end
         end
-        res = call_same_with_inverted_arg_if_active!(B, gutils, orig, args2, valTys, lookup; kwargs..., movebefore=idx == 1)
+        res = call_same_with_inverted_arg_if_active!(B, gutils, orig, args2, valTys, lookup; need_result, kwargs..., movebefore=idx == 1)
         if shadow === nothing
             continue
         end
@@ -357,4 +362,4 @@ function batch_call_same_with_inverted_arg_if_active!(
     end
 
     return shadow
-end
\ No newline at end of file
+end
