@@ -26,6 +26,8 @@ const nofreefns = Set{String}((
     "jl_f_isdefined",
     "ijl_field_index",
     "jl_field_index",
+    "ijl_get_field_offset",
+    "jl_get_field_offset",
     "ijl_specializations_get_linfo",
     "jl_specializations_get_linfo",
     "ijl_gf_invoke_lookup_worlds",
@@ -107,6 +109,8 @@ const nofreefns = Set{String}((
     "jl_get_ptls_states",
     "ijl_get_ptls_states",
     "jl_f_fieldtype",
+    "jl_f_sizeof",
+    "ijl_f_sizeof",
     "jl_symbol_n",
     "jl_stored_inline",
     "ijl_stored_inline",
@@ -178,6 +182,8 @@ const inactivefns = Set{String}((
     "jl_f_isdefined",
     "ijl_field_index",
     "jl_field_index",
+    "ijl_get_field_offset",
+    "jl_get_field_offset",
     "ijl_specializations_get_linfo",
     "jl_specializations_get_linfo",
     "ijl_gf_invoke_lookup_worlds",
@@ -256,6 +262,8 @@ const inactivefns = Set{String}((
     "memhash_seed",
     "jl_f__typevar",
     "ijl_f__typevar",
+    "jl_f_sizeof",
+    "ijl_f_sizeof",
     "jl_f_isa",
     "ijl_f_isa",
     "jl_set_task_threadpoolid",
@@ -739,14 +747,14 @@ function annotate!(mod::LLVM.Module)
         "ijl_gc_alloc_typed",
         "jl_alloc_genericmemory",
         "ijl_alloc_genericmemory",
+	"jl_alloc_genericmemory_unchecked",
+	"ijl_alloc_genericmemory_unchecked",
         "jl_alloc_array_1d",
         "jl_alloc_array_2d",
         "jl_alloc_array_3d",
         "ijl_alloc_array_1d",
         "ijl_alloc_array_2d",
         "ijl_alloc_array_3d",
-        "jl_alloc_genericmemory",
-        "ijl_alloc_genericmemory",
         "ijl_new_array",
         "jl_new_array"
     )
@@ -800,6 +808,8 @@ function annotate!(mod::LLVM.Module)
         "ijl_box_int64",
         "jl_alloc_genericmemory",
         "ijl_alloc_genericmemory",
+	"jl_alloc_genericmemory_unchecked",
+	"ijl_alloc_genericmemory_unchecked",
         "jl_alloc_array_1d",
         "jl_alloc_array_2d",
         "jl_alloc_array_3d",
@@ -812,8 +822,6 @@ function annotate!(mod::LLVM.Module)
         "ijl_genericmemory_slice",
         "jl_genericmemory_copy_slice",
         "ijl_genericmemory_copy_slice",
-        "jl_alloc_genericmemory",
-        "ijl_alloc_genericmemory",
         "jl_idtable_rehash",
         "ijl_idtable_rehash",
         "jl_f_tuple",
