@@ -155,13 +155,12 @@ struct ReversePFor{ThunkTy, FT, AnyJL, byRef, TT}
 end
 
 function (st::ReversePFor{ThunkTy, FT, AnyJL, byRef, TT})(tid) where {ThunkTy, FT, AnyJL, byRef, TT}
-
     tres = if !AnyJL
         unsafe_load(st.tapes, tid)
     else
         @inbounds st.tapes[tid]
     end
-
+    
     if byRef
         st.thunk(Const(referenceCaller), st.ft, Const(tid), tres)
     else
@@ -239,8 +238,18 @@ end
     width = Int(get_width(gutils))
 
     ops = collect(operands(orig))[1:end-1]
-    dupClosure =
-        !guaranteed_const_nongen(funcT, world) && !is_constant_value(gutils, ops[1])
+    dupClosure = !guaranteed_const_nongen(funcT, world)
+    if dupClosure
+	if is_constant_value(gutils, ops[1])
+	    dupClosure = false
+	    if inline_roots_type(funcT) != 0
+	        if !is_constant_value(gutils, ops[2])
+		    dupClosure = true
+		end
+	    end
+	end
+    end
+  
     pdupClosure = dupClosure
 
     subfunc = nothing
@@ -414,28 +423,50 @@ end
     al = addrspacecast!(B, al, LLVM.PointerType(ll_th, Tracked))
     al = addrspacecast!(B, al, LLVM.PointerType(ll_th, Derived))
     push!(vals, al)
+    @assert inline_roots_type(thunkTy) == 0
 
     copies = Tuple{LLVM.Value, LLVM.Value, LLVM.LLVMType}[]
     if !isghostty(dfuncT)
 
         llty = convert(LLVMType, dfuncT)
 
-        alloctx = LLVM.IRBuilder()
+	num_arg_roots = inline_roots_type(llty)
+        
+	alloctx = LLVM.IRBuilder()
         position!(alloctx, LLVM.BasicBlock(API.EnzymeGradientUtilsAllocationBlock(gutils)))
         al = alloca!(alloctx, llty)
+	al2 = if num_arg_roots != 0
+	   alloca!(alloctx, convert(LLVMType, AnyArray(num_arg_roots)))
+	end
 
         if !isghostty(ppfuncT)
             v = new_from_original(gutils, ops[1])
+            pllty = convert(LLVMType, ppfuncT)
+	    
+            pv = nothing
+                
+	    fwdbuilder = if mode == API.DEM_ReverseModeGradient
+	       B2 = LLVM.IRBuilder()
+	       position!(B2, new_from_original(gutils, orig))
+	       B2
+	    else
+	       B
+	    end
+	    
+            if value_type(v) != pllty
+                pv = v
+                v = load!(fwdbuilder, pllty, v)
+            end
+	    
+	    if inline_roots_type(ppfuncT) != 0
+		v2 = new_from_original(gutils, ops[2])
+		v = recombine_value!(fwdbuilder, v, v2)
+	    end
+
             if mode == API.DEM_ReverseModeGradient
                 v = lookup_value(gutils, v, B)
             end
 
-            pllty = convert(LLVMType, ppfuncT)
-            pv = nothing
-            if value_type(v) != pllty
-                pv = v
-                v = load!(B, pllty, v)
-            end
         else
             v = makeInstanceOf(B, ppfuncT)
         end
@@ -443,12 +474,14 @@ end
         if refed
             val0 = val = emit_allocobj!(B, pfuncT)
             val = bitcast!(B, val, LLVM.PointerType(pllty, addrspace(value_type(val))))
-            val = addrspacecast!(B, val, LLVM.PointerType(pllty, Derived))
-            store!(B, v, val)
-            if !(pv isa Nothing)
+            val = addrspacecast!(B, val, LLVM.PointerType(pllty, Derived)) 
+
+	        if !(pv isa Nothing)
                 push!(copies, (pv, val, pllty))
             end
 
+            store!(B, v, val)
+
             if any_jltypes(pllty)
                 emit_writebarrier!(B, get_julia_inner_types(B, val0, v))
             end
@@ -462,32 +495,61 @@ end
             al,
             [LLVM.ConstantInt(LLVM.IntType(64), 0), LLVM.ConstantInt(LLVM.IntType(32), 0)],
         )
+	    
+        if al2 !== nothing
+           extract_roots_from_value!(B, val0, al2)
+           T_jlvalue = LLVM.StructType(LLVMType[])
+           T_prjlvalue = LLVM.PointerType(T_jlvalue, Tracked)
+           al3 = gep!(B, T_prjlvalue, al2, LLVM.Value[ConstantInt(CountTrackedPointers(value_type(val0)).count)])
+        end
+            
         store!(B, val0, ptr)
 
         if pdupClosure
 
             if !isghostty(ppfuncT)
                 dv = invert_pointer(gutils, ops[1], B)
-                if mode == API.DEM_ReverseModeGradient
-                    dv = lookup_value(gutils, dv, B)
-                end
-
+                   
+		fwdbuilder = if mode == API.DEM_ReverseModeGradient
+		     B2 = LLVM.IRBuilder()
+		     position!(B2, new_from_original(gutils, orig))
+		     B2
+		   else
+		     B
+		   end
+	        
                 spllty = LLVM.LLVMType(API.EnzymeGetShadowType(width, pllty))
                 pv = nothing
+	        
+                dv2 = if inline_roots_type(ppfuncT) != 0
+                   invert_pointer(gutils, ops[2], B)
+                end
+
                 if value_type(dv) != spllty
+                    pv = dv
                     if width == 1
-                        pv = dv
-                        dv = load!(B, spllty, dv)
+                        dv = load!(fwdbuilder, spllty, dv)
+			            if dv2 !== nothing
+                           dv = recombine_value!(fwdbuilder, dv, dv2)
+                        end
                     else
                         shadowres = UndefValue(spllty)
                         for idx = 1:width
-                            arg = extract_value!(B, dv, idx - 1)
-                            arg = load!(B, pllty, arg)
-                            shadowres = insert_value!(B, shadowres, arg, idx - 1)
+                            arg = extract_value!(fwdbuilder, dv, idx - 1)
+                            arg = load!(fwdbuilder, pllty, arg)
+                            if dv2 !== nothing
+                              arg2 = extract_value!(fwdbuilder, dv2, idx - 1)
+                              arg = recombine_value!(fwdbuilder, arg, arg2)
+                            end
+                            shadowres = insert_value!(fwdbuilder, shadowres, arg, idx - 1)
                         end
                         dv = shadowres
                     end
                 end
+                
+                if mode == API.DEM_ReverseModeGradient
+                    dv = lookup_value(gutils, dv, B)
+                end
             else
                 @assert false
             end
@@ -498,12 +560,24 @@ end
                     bitcast!(B, dval, LLVM.PointerType(spllty, addrspace(value_type(dval))))
                 dval = addrspacecast!(B, dval, LLVM.PointerType(spllty, Derived))
                 store!(B, dv, dval)
-                if pv !== nothing
-                    push!(copies, (pv, dval, spllty))
-                end
                 if any_jltypes(spllty)
                     emit_writebarrier!(B, get_julia_inner_types(B, dval0, dv))
                 end
+                pvl = lookup_value(gutils, pv, B)
+                if mode == API.DEM_ReverseModeGradient
+                    if width == 1
+                       copy_floats_into!(B, spllty, dval, pvl)
+                    else
+                       for idx = 1:width
+                           arg = extract_value!(B, pvl, idx - 1)
+                           g0 = inbounds_gep!(B, spllty, dval, LLVM.Value[LLVM.ConstantInt(Int64(0)), LLVM.ConstantInt(Int32(idx-1))])
+                           copy_floats_into!(B, pllty, g0, arg)
+                        end
+                    end
+                end
+                if pv !== nothing
+                    push!(copies, (pv, dval, spllty))
+                end
             else
                 dval0 = dv
             end
@@ -517,12 +591,21 @@ end
                     LLVM.ConstantInt(LLVM.IntType(32), 1),
                 ],
             )
+	
+	    if al2 !== nothing
+	       extract_roots_from_value!(B, dval0, al3)
+	    end
             store!(B, dval0, dptr)
         end
 
         al = addrspacecast!(B, al, LLVM.PointerType(llty, Derived))
 
         push!(vals, al)
+        
+	if num_arg_roots != 0
+	  push!(vals, al2)
+
+	end
     end
 
     if tape !== nothing
@@ -553,7 +636,10 @@ end
     entry = nested_codegen!(mode, mod, runtime_pfor_fwd, tt, world)
     push!(function_attributes(entry), EnumAttribute("alwaysinline"))
 
-    pval = const_ptrtoint(functions(mod)[sname], convert(LLVMType, Ptr{Cvoid}))
+    pval = functions(mod)[sname]
+    if VERSION < v"1.12"
+        pval = const_ptrtoint(pval, convert(LLVMType, Ptr{Cvoid}))
+    end
     pval = LLVM.ConstantArray(value_type(pval), [pval])
     store!(B, pval, vals[1])
 
@@ -565,6 +651,7 @@ end
         unsafe_store!(normalR, C_NULL)
     else
         ni = new_from_original(gutils, orig)
+	API.EnzymeReplaceOriginalToNew(gutils, orig, cal)
         API.EnzymeGradientUtilsErase(gutils, ni)
     end
     return false
@@ -597,7 +684,10 @@ end
     entry = nested_codegen!(mode, mod, runtime_pfor_augfwd, tt, world)
     push!(function_attributes(entry), EnumAttribute("alwaysinline"))
 
-    pval = const_ptrtoint(functions(mod)[sname], convert(LLVMType, Ptr{Cvoid}))
+    pval = functions(mod)[sname]
+    if VERSION < v"1.12"
+       pval = const_ptrtoint(pval, convert(LLVMType, Ptr{Cvoid}))
+    end
     pval = LLVM.ConstantArray(value_type(pval), [pval])
     store!(B, pval, vals[1])
 
@@ -616,6 +706,7 @@ end
         unsafe_store!(normalR, C_NULL)
     else
         ni = new_from_original(gutils, orig)
+	API.EnzymeReplaceOriginalToNew(gutils, orig, tape)
         API.EnzymeGradientUtilsErase(gutils, ni)
     end
 
@@ -652,7 +743,10 @@ end
     entry = nested_codegen!(mode, mod, runtime_pfor_rev, tt, world)
     push!(function_attributes(entry), EnumAttribute("alwaysinline"))
 
-    pval = const_ptrtoint(functions(mod)[sname], convert(LLVMType, Ptr{Cvoid}))
+    pval = functions(mod)[sname]
+    if VERSION < v"1.12"
+	pval = const_ptrtoint(pval, convert(LLVMType, Ptr{Cvoid}))
+    end
     pval = LLVM.ConstantArray(value_type(pval), [pval])
     store!(B, pval, vals[1])
 
@@ -661,6 +755,7 @@ end
 
     for (pv, val, pllty) in copies
         ld = load!(B, pllty, val)
+	pv = lookup_value(gutils, pv, B)
         store!(B, ld, pv)
     end
     return nothing
